#!/usr/bin/env python
# vim: set sts=4 expandtab:
"""
moin2ascii - convert a moinmoin text file to an AsciiDoc

Copyright (C) 2009 Osamu Aoki, GPL
"""

import sys, os, re

VERSION = '1.0.1'

def process_text(line):
#    line = re.sub(r"\\",'@bslash@',line)
    line = re.sub(r"\\",r'\\',line)
    line = re.sub(r"\~",r'\~',line)
    line = re.sub(r"\^",r'\^',line)
    line = re.sub(r"\*",r'\*',line)
    line = re.sub(r'{{{',r'`',line)
    line = re.sub(r'}}}',r'`',line)
    line = re.sub(r"'''",r'**',line)
    line = re.sub(r"''",r'__',line)
    line = re.sub(r"'",r"\'",line)
#    line = re.sub(r"\&",r'&amp;',line)
#    line = re.sub(r"\<",r'&gt;',line)
#    line = re.sub(r"\>",r'&lt;',line)
    line = re.sub(r'\[\[([^|]+)\|([^]]+)\]\]',r'\1[\2]',line)
    line = re.sub(r'\[\[([^|]+)\\]\]',r'\1[\1]',line)
    return line

def process_indent(line):
    if re.match(r'^=',line):
        line = re.sub(r'^=(.*) =+',r'==\1',line)
    elif line[0:2] == ' *':
        line = "-" + line[2:]
    elif line[0:3] == '  *':
        line = "--" + line[3:]
    elif re.match(r'^ +[0-9]\.',line):
        line = line[1:]
    elif line[0:3] == '/!\\':
        line = "WARNING:" + line[3:]
    elif line[0:3] == r'<!>':
        line = "CAUTION:" + line[3:]
    elif line[0:3] == r'{i}':
        line = "TIP:" + line[3:]
    elif line[0:3] == r'(!)':
        line = "NOTE:" + line[3:]
    line =  process_text(line)
    if line[0:2] == '--':
        line = " *" + line[2:]
    return line

def process_table(line):
    line = re.sub(r'\|\| *([^ ].*[^ ]) *\|\|',r'\1',line)
    items = re.split(r' *\|\| *',line,8)
    i = 0
    for item in items:
        items[i] = process_text(item)
        i = i + 1
    return items

if __name__ == '__main__':

    m2a_mode = 'NORMAL'
    m2a_table_mode = 0
    m2a_table_popcon = 0

    for line in sys.stdin.readlines():
        line=line[0:-1]
        if line == r"{{{#!plain":
            line = "{{{"
        if m2a_mode != 'SCREEN':
            if line[0:2] == '##':
                print "//" + line[2:]
            elif line[0:2] == '||':
                if m2a_mode == 'NORMAL':
                    table_items = [[]]
                    table_rows = 0
                    table_cols = 0
                    table_cols_width = [0]
                    table_item =  process_table(line)
                    table_title = table_item[0]
                    if len(table_item) != 1:
                        table_pkg = table_item[1] # '', '1', '2'
                    else:
                        table_pkg =0
                    print "\n." + table_title
                    print '[grid="all"]'
                    m2a_mode = 'TABLE_HEAD'
                elif m2a_mode == 'TABLE_HEAD':
                    line = re.sub("'''",'',line)
                    table_items =  [process_table(line)]
                    i = 0
                    m2a_mode = 'TABLE_BODY'
                elif m2a_mode == 'TABLE_BODY':
                    table_item =  process_table(line)
                    if table_pkg == "1":
                        table_item[1] = "@@@popcon1@@@"
                        table_item[2] = "@@@psize1@@@"
                    if table_pkg == "2":
                        table_item[2] = "@@@popcon2@@@"
                        table_item[3] = "@@@psize2@@@"
                    table_items = table_items + [table_item]
                    m2a_mode = 'TABLE_BODY'
            else:
                if m2a_mode == 'TABLE_BODY':
                    # now time to print table
                    table_rows = len(table_items)
                    # find table_cols_width
                    table_cols_width = [0,0,0,0,0,0,0,0,0]
                    for table_item in table_items:
                        i = 0
                        for item in table_item:
                            table_cols_width[i] = max(table_cols_width[i], len(item))
                            i = i + 1
                    # print ruler
                    ruler1 = ""
                    ruler2 = ""
                    i = 0
                    for item in table_cols_width:
                        if item != 0:
                            ruler1 = ruler1 + "`" + ( "-" * item )
                            ruler2 = ruler2 + "-" + ( "-" * item )
                    print ruler1
                    # print table_items
                    j = 0
                    for table_item in table_items:
                        i = 0
                        for item in table_item:
                            print item + " " * (table_cols_width[i] - len(item)),
                            i = i + 1
                        print ""
                        j = j + 1
                        if j == 1:
                            print ruler2
                    print ruler2

                    m2a_mode = 'NORMAL'

                # so this is normal text
                if re.match("^{{{$", line):
                    print ''
                    print '--------------------'
                    m2a_mode = 'SCREEN'
                elif re.match("^ +{{{$", line):
                    print ' +\n--------------------'
                    m2a_mode = 'SCREEN'
                else:
                    if re.match("^ +\*", line):
                        if m2a_mode != 'BULLET':
                            print ""
                            m2a_mode = 'BULLET'
                    else:
                            m2a_mode = 'NORMAL'
                    print process_indent(line)
        else:
            if re.match("^}}}$", line):
                print '--------------------'
                m2a_mode = 'NORMAL'
            else:
                print line
                m2a_mode = 'SCREEN'

""" 
Design: Convert text depending on mode:

MODE = NORMAL
-------------

This is default mode.
This can be base text including buletted/numerated list items.

Processing:
+ Ajust header leads: "s/^=/==/"
+ Adjust buletted/numerated list items.
+ /!\ for "WARNING"
+ <!> for "CAUTION"
+ {i} for "TIP"
+ (!) for "NOTE"
+ content text adjustments and processing

MODE = TABLE
------------

This is table section.
It starts at a line matching "^||"
It ends at a line matching "^[^|][^|]"

Processing:
+ The first line is translated to "TITLE of TABLE".
+ .. If first line contains 2,3,4th argument, they are processed (popcon/size).
     Insert: @@@popcon@@@, @@@size@@@
+ The second line is translated to "HEADER".
+ content text adjustments and processing

MODE = SCREEN
-------------

This is screen section.
It starts at a line matching "^{{{" or "^ {{{"
It ends at a line matching "^}}}"

Processing:
+ verbatim copy as screen ....
+ In case of "^ {{{", continuation of indented list item.

process_line()
------------------
Content text adjustments and processing inclises folowing
+ format converting
 * URL: [[http://...|...]]          --> http://...[...]
 * URL: [[http://...]]              --> http://...
 * The verbatim text are {{{....}}} --> +++....++++
 * ''' for strong *
 * ''  for italics _
 * @{@...@}@ to new style (predefined)
+ reporting to stderr
 * normal text with funny strings
 * @{@...@}@ unprocessed
"""



#!/usr/bin/env python
# vim: set sts=4 expandtab:
"""
po4a-old

Copyright (C) 2009 Osamu Aoki, GPL

Design: reverse msgid of po file to old one

Prerequisite: msgmerge --previous --no-wrap

"""

import sys, os, re

VERSION = '1.0.1'

msgold = ''
if __name__ == '__main__':

    for line in sys.stdin.readlines():
        # drop EOL = NL
        line=line[0:-1]
        if re.match("^[ \t]*$", line):
            # reformat as clean blank line
            print line
        elif re.match("^#. msgid ", line):
            msgold = re.sub(r"^#. msgid +\"(.*)\" *$",r'\1',line)
            print line
        elif re.match("^msgid ", line):
            msgnew = re.sub(r"^msgid +\"(.*)\" *$",r'\1',line)
            if msgold == '':
                print line
            else:
                print "msgid \"" + msgold + "\""
            # reset
            msgold = ''
        elif re.match("^msgstr ", line):
            msgstr = re.sub(r"^msgstr +\"(.*)\" *$",r'\1',line)
            print line
        elif re.match("^#", line):
            # comment line
            print line
        else:
            print line
        #





#!/usr/bin/python2
# vim: set sts=4 expandtab:

"""
input:
.title
[grid="all"]
`-------------`------------`------`--------`-----------------------------------------
file          permission   user   group    description
-------------------------------------------------------------------------------------
`/etc/passwd` `-rw-r--r--` `root` `root`   (sanitized) user account information
`/etc/shadow` `-rw-r-----` `root` `shadow` secure user account information
`/etc/group`  `-rw-r--r--` `root` `root`   group information
-------------------------------------------------------------------------------------

output:
.title
[width="100%", cols="20,12,6,12,50",grid="all",options="header"]
|=====================================================================================
|file          |permission   |user   |group    |description
|`/etc/passwd` |`-rw-r--r--` |`root` |`root`   |(sanitized) user account information
|`/etc/shadow` |`-rw-r-----` |`root` |`shadow` |secure user account information
|`/etc/group`  |`-rw-r--r--` |`root` |`root`   |group information
|=====================================================================================

"""

import sys, os, re, string

VERSION = '1.0.1'

def main():
    flag_table  = 0 # not in table
    flag_example  = 0 # not in example
    maxlength = 512
    index = 0
    for line in sys.stdin.readlines():
        line=line[0:-1]
        index += 1
        if flag_table == 0 and flag_example == 0 and re.match('\[grid', line):
            # pre-table header
            pass # do nothing
        elif flag_table == 0 and flag_example == 0 and re.match('`---', line):
            # 1st line in table
            flag_table   = 1 # now in ruler line of table
            i = 0 # sweeping pointer
            j = 0 # previous pointer
            pos = []
            column = 1
            for x in line:
                if x == '`' and i > 0:
                    pos.append((j, i))
                    column += 1
                    j = i
                i += 1
            pos.append((j, maxlength))
            if column == 1:
                print >> sys.stderr, "ETC1: column = 1 at", index
                sys.exit(1)
            elif column == 2:
                print '[width="100%", cols="40,60", grid="all", options="header"]'
            elif column == 3:
                line = '[width="100%", cols="30,30,40", grid="all", options="header"]'
            elif column == 4:
                print '[width="100%", cols="20,12,6,62", grid="all", options="header"]'
            elif column == 5:
                print '[width="100%", cols="20,12,6,12,50", grid="all", options="header"]'
            elif column == 6:
                print '[width="100%", cols="20,12,6,12,12,38", grid="all", options="header"]'
            elif column == 7:
                print '[width="100%", cols="20,12,6,12,12,12,26", grid="all", options="header"]'
            else:
                print >> sys.stderr, "ETC8: column > 7 at", index
                sys.exit(1)
            print '|=============================================================='
        elif flag_table == 1 and flag_example == 0:
            # 2nd line in table
            flag_table   = 2 # now in table header
            # process data entry
            xline = ""
            for x in pos:
                i, j = x
                if j < maxlength:
                    xline = xline + "|" + re.sub('\|', '@@@bar@@@',line[i:j].strip())
                else:
                    xline = xline + "|" + re.sub('\|', '@@@bar@@@',line[i:].strip())
            print xline
        elif flag_table == 2 and flag_example == 0:
            # 3rd line in table: do not output
            flag_table   = 3 # now in table header and contents separator
            if not re.match('----', line):
                print >> sys.stderr, "ET4: should be table header and contents separator at", index
                sys.exit(1)
            pass # do nothing
        elif flag_table > 2 and flag_example == 0 and not re.match('----', line):
            # 4th line or later in table but not last
            flag_table   += 1 # now in table contents
            # process data entry
            xline = ""
            for x in pos:
                i, j = x
                if j < maxlength:
                    xline = xline + "|" + re.sub('\|', '@@@bar@@@',line[i:j])
                else:
                    xline = xline + "|" + re.sub('\|', '@@@bar@@@',line[i:])
            print xline
        elif flag_table > 2 and flag_example == 0 and re.match('----', line):
            # last line in table
            flag_table   = 0 # now out of table contents
            print '|=============================================================='
        elif flag_table == 0 and flag_example == 0 and re.match('----', line):
            flag_example = 1
            print '---------------------------------------------------------------'
        elif flag_table == 0 and flag_example == 1 and re.match('----', line):
            flag_example = 0
            print '---------------------------------------------------------------'
        elif flag_table == 0 and flag_example == 1:
            print line.rstrip()
        elif flag_table == 0 and flag_example == 0:
            print line.rstrip()
        else:
            print >> sys.stderr, "ETX: strange at", index, flag_table
            sys.exit(1)

if __name__ == '__main__':
    main()



#!/usr/bin/env python
# vim: set sts=4 expandtab:

"""
colspec.py - set colspec in XML to correct values

Copyright (C) 2009 Osamu Aoki, GPL
"""

import sys, os, re

VERSION = '1.0.1'

if __name__ == '__main__':

    table_id  = ''
    tgroup_cols = 0

    for line in sys.stdin.readlines():
        line=line[0:-1]
        if re.match('.*table id="', line):
            table_id = re.sub(r'.*table id="([^"]*)".*',r'\1',line)
#            print "table id =" + table_id 
        elif re.match('.*<title>', line):
            title = re.sub(r'.*<title>([^<]*)</title>.*',r'\1',line)
#            print "title =" + title
        elif re.match('.*tgroup cols="', line):
            tgroup_cols = int(re.sub(r'.*tgroup cols="([^"]*)".*',r'\1',line))
#            print "tgroup_cols =", tgroup_cols
            cols = 0
            if tgroup_cols == 2:
                colwidth = (40,60)
            elif tgroup_cols == 3:
                colwidth = (30,30,40)
            elif tgroup_cols == 4:
                colwidth = (20,12,6,62)
            elif tgroup_cols == 5:
                colwidth = (20,12,6,12,50)
            elif tgroup_cols == 6:
                colwidth = (20,12,6,12,12,38)
            elif tgroup_cols == 7:
                colwidth = (20,12,6,12,12,12,26)
            # List of exceptions
            if table_id == "thenumericmodefoinchmodbcommands":
                colwidth = (20,80)
            elif table_id == "listofnotablesysupsforfileaccess":
                colwidth = (20,80)
            elif table_id == "listofnotablesysommandexecutions":
                colwidth = (20,80)
            elif table_id == "listoftypesoftimestamps":
                colwidth = (20,80)
            elif table_id == "listofnotablesysupsforfileaccess":
                colwidth = (20,80)
            elif table_id == "listofnotablepinpinningtechnique":
                colwidth = (20,80)
            elif table_id == "listofkeyboardreigurationmethods":
                colwidth = (20,80)
            elif table_id == "listoffilesystemalusagescenarios":
                colwidth = (20,80)
            elif table_id == "thereplacementexpression":
                colwidth = (30,70)
            elif table_id == "thekeybindingsofmc":
                colwidth = (30,70)
            elif table_id == "listofkeywebsiteaspecificpackage":
                colwidth = (30,70)
            elif table_id == "listofspecialdevicefiles":
                colwidth = (30,20,50)
            elif table_id == "listofdebianarchivearea":
                colwidth = (20,20,60)
            elif table_id == "listofviewsforaptitude":
                colwidth = (20,20,60)
            elif table_id == "listofrunlevelsationoftheirusage":
                colwidth = (20,20,60)
            elif table_id == "listofsshauthenttocolsandmethods":
                colwidth = (20,40,40)
            elif table_id == "listofterminologornetworkdevices":
                colwidth = (20,20,25,35)
            elif table_id == "tableofkeywordsundicatefonttypes":
                colwidth = (25,25,25,25)
            elif table_id == "listofinsecureanservicesandports":
                colwidth = (25,25,25,25)
            elif table_id == "therelationshipbsuiteandcodename":
                colwidth = (25,25,25,25)
            elif table_id == "theumaskvalueexamples":
                colwidth = (20,25,25,30)
            elif table_id == "dimportantconfigilesforpam_unixi":
                colwidth = (20,20,10,10,40)
            elif table_id == "listoftoolstogeneratepassword":
                colwidth = (15,15,10,15,45)
            elif table_id == "listofnetworkaddressranges":
                colwidth = (10,40,30,10,10)


        if re.match('.*table pgwide="', line):
            print re.sub(r'(.*table pgwide=)"0"(.*)',r'\1"1"\2',line)
        elif re.match('.*colspec colwidth="', line):
            x = str(colwidth[cols])
            cols += 1
            print re.sub(r'(.*colspec colwidth=").*',r'\1',line)+str(x)+"*"+re.sub(r'.*colspec colwidth="[^"]*(".*)',r'\1',line)
        else:
            print line

